unit Unit1;

{$mode objfpc}{$H+}

// http://nehe.gamedev.net/

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, OpenGLContext,
  //dglOpenGL;

{ statt der dglOpenGL können auch die units GL, GLU eingebunden werden
  Bei Verwednung der dglOpenGL unit muss beim Form.Create eine saubere
  Initialisierung erfolgen }
  GL, GLU;

type

  { TForm1 }

  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure OnAppIdle(Sender: TObject; var Done: Boolean);
    procedure OpenGLControl1Paint(Sender: TObject);
    procedure OpenGLControl1Resize(Sender: TObject);
  private
    OpenGLControl1: TOpenGLControl;
    procedure Kugel(Radius: Double; Slices, Stacks: Integer);
  public

  end;

var
  Form1: TForm1;
  rtri: GLFloat = 0.0;
  rquad: GLFloat = 0.0;
  EarthRot, MoonRot, EarthSpin : Double;  // Rotationsvariablen
  ViewerPosX: GLFloat = -100;
  ViewerPosY: GLFloat = 60;
  ViewerPosZ: GLFloat = 40;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  OpenGLControl1:=TOpenGLControl.Create(Self);
  with OpenGLControl1 do begin
    Name:='OpenGLControl1';
    Align:=alClient;
    Parent:=Self;
    OnPaint:=@OpenGLControl1Paint;
    OnResize:=@OpenGLControl1Resize;
    AutoResizeViewport:=true;
  end;

  { Beginn: Initialisierung von OpenGL bei Verwendung der dglOpenGL}
  //InitOpenGL;
  OpenGLControl1.MakeCurrent;
  //ReadExtensions;
  //ReadImplementationProperties;
  { End: Initialisierung von OpenGL bei Verwendung der dglOpenGL}

  { Beginn: Initialisierung von OpenGL gemäß Neon Helium}
  glEnable(GL_TEXTURE_2D);             // Aktiviert Texture Mapping
  glShadeModel(GL_SMOOTH);             // Aktiviert weiches Shading
  glClearColor(0.0, 0.0, 0.0, 0.5);    // Bildschirm löschen
  glClearDepth(1.0);                   // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);             // Aktiviert Despth Testing
  glDepthFunc(GL_LEQUAL);              // Type of Depth Test to Do
  glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST); // Qualitativ bessere Koordinaten Interpolation
  { End: Initialisierung von OpenGL gemäß Neon Helium}


  { Aufruf des IdleHandlers für das kontinuierliche Durchlaufen der .FormPaint }
  Application.AddOnIdleHandler(@OnAppIdle);
end;

procedure TForm1.OnAppIdle(Sender: TObject; var Done: Boolean);
begin
  Done:=false;
  //DebugLn(['TForm1.OnAppIdle ']);
  OpenGLControl1.Invalidate;
end;

procedure TForm1.Kugel(Radius: Double; Slices, Stacks: Integer);
var
  q: PGLuquadric;
begin
  q := gluNewQuadric();
  gluQuadricOrientation(q, glu_outside);
  gluQuadricTexture(q, gl_false);
  gluQuadricDrawStyle(q, GLU_LINE);
  glShademodel(gl_flat);
  gluQuadricTexture(q, gl_false);
  gluSphere(q, Radius, Slices, Stacks);
  gluDeleteQuadric(q);
end;

procedure TForm1.OpenGLControl1Paint(Sender: TObject);
begin
  // Screen- und Tiefenbuffer bereinigen
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT ); // Clear the Screen and the Depth

  EarthRot:=EarthRot+3/365;
  MoonRot:=MoonRot+3/60;
  EarthSpin := EarthSpin + 200/365;

  glMatrixMode(GL_MODELVIEW); // Zvolí matici Modelview
  glLoadIdentity; // Reset the Current ModelView Matrix
  gluLookAt(100, 30, 20, 0,0,0, 0,1,0);

  // Koordinatensystem
  glBegin  (GL_Lines);
    glColor3f(1,0,0);  // x --> rot
    glVertex3f(0,0,0);
    glVertex3f(50,0,0);
    glColor3f(0,1,0);  // y --> grün
    glVertex3f(0,0,0);
    glVertex3f(0,50,0);
    glColor3f(0,0,1);  // z --> blau
    glVertex3f(0,0,0);
    glVertex3f(0,0,50);
  glEnd;

  // Sonne
  glEnable(gl_normalize);
  glLineWidth(1);
  glColor3f(1,1,0);
  Kugel(10, 15, 15);

  // Erde + Mond
  glRotated(EarthRot,0, 1,0);
  gltranslated(30, 0, 0);

  glPushMatrix;

    // Erde
    glPushmatrix;
      glRotated(EarthSpin, 0, 1, 0);  // --> Eigenrotation Erde
      glColor3f(0,0,1);
      Kugel(5, 20, 20);
    glPopMatrix;

    //Mond
    glRotated(MoonRot, 0,1,0);
    gltranslated(8,0,0);
    glColor3f(0,1,1);
    Kugel(1, 10, 10);
  glPopMatrix;

  OpenGLControl1.SwapBuffers;
end;


procedure TForm1.OpenGLControl1Resize(Sender: TObject);
begin
  if OpenGLControl1.Height <= 0 then exit;
  // the viewport is automatically resized by the TOpenGLControl
  // you can disable it (OpenGLControl1.AutoResizeViewport:=false)
  // and do something yourself here

  glViewport(0, 0, Width, Height);     // Setzt den Viewport für das OpenGL Fenster
                                       // Width & Height existieren in der UNIT CONTROLS

  glMatrixMode(GL_PROJECTION);         // Matrixmode auf Projektion setzen
  glLoadIdentity();                    // und reset der View
  gluPerspective(45.0,Width/Height,0.1,150.0); // Výpoèet perspektivy
  glViewport(0, 0, Width, Height); // Resetuje aktuální nastavení

  glMatrixMode(GL_MODELVIEW); // Zvolí matici Modelview
  glLoadIdentity; // Reset the Current ModelView Matrix
end;

end.


